from ovos_utils.messagebus import Message, get_mycroft_bus


class EnclosureAPI:
    """
    This API is intended to be used to interface with the hardware
    that is running Mycroft.  It exposes all possible commands which
    can be sent to a Mycroft enclosure implementation.

    Different enclosure implementations may implement this differently
    and/or may ignore certain API calls completely.  For example,
    the eyes_color() API might be ignore on a Mycroft that uses simple
    LEDs which only turn on/off, or not at all on an implementation
    where there is no face at all.
    """

    def __init__(self, bus=None):
        self.bus = bus or get_mycroft_bus()

    def reset(self):
        """The enclosure should restore itself to a started state.
        Typically this would be represented by the eyes being 'open'
        and the mouth reset to its default (smile or blank).
        """
        self.bus.emit(Message("enclosure.reset"))

    def system_reset(self):
        """The enclosure hardware should reset any CPUs, etc."""
        self.bus.emit(Message("enclosure.system.reset"))

    def system_mute(self):
        """Mute (turn off) the system speaker."""
        self.bus.emit(Message("enclosure.system.mute"))

    def system_unmute(self):
        """Unmute (turn on) the system speaker."""
        self.bus.emit(Message("enclosure.system.unmute"))

    def system_blink(self, times):
        """The 'eyes' should blink the given number of times.
        Args:
            times (int): number of times to blink
        """
        self.bus.emit(Message("enclosure.system.blink", {'times': times}))

    def eyes_on(self):
        """Illuminate or show the eyes."""
        self.bus.emit(Message("enclosure.eyes.on"))

    def eyes_off(self):
        """Turn off or hide the eyes."""
        self.bus.emit(Message("enclosure.eyes.off"))

    def eyes_blink(self, side):
        """Make the eyes blink
        Args:
            side (str): 'r', 'l', or 'b' for 'right', 'left' or 'both'
        """
        self.bus.emit(Message("enclosure.eyes.blink", {'side': side}))

    def eyes_narrow(self):
        """Make the eyes look narrow, like a squint"""
        self.bus.emit(Message("enclosure.eyes.narrow"))

    def eyes_look(self, side):
        """Make the eyes look to the given side
        Args:
            side (str): 'r' for right
                        'l' for left
                        'u' for up
                        'd' for down
                        'c' for crossed
        """
        self.bus.emit(Message("enclosure.eyes.look", {'side': side}))

    def eyes_color(self, r=255, g=255, b=255):
        """Change the eye color to the given RGB color
        Args:
            r (int): 0-255, red value
            g (int): 0-255, green value
            b (int): 0-255, blue value
        """
        self.bus.emit(Message("enclosure.eyes.color",
                              {'r': r, 'g': g, 'b': b}))

    def eyes_setpixel(self, idx, r=255, g=255, b=255):
        """Set individual pixels of the Mark 1 neopixel eyes
        Args:
            idx (int): 0-11 for the right eye, 12-23 for the left
            r (int): The red value to apply
            g (int): The green value to apply
            b (int): The blue value to apply
        """
        if idx < 0 or idx > 23:
            raise ValueError('row ({}) must be between 0-23'.format(str(idx)))
        self.bus.emit(Message("enclosure.eyes.setpixel",
                              {'row': idx, 'r': r, 'g': g, 'b': b}))

    def eyes_fill(self, percentage):
        """Use the eyes as a type of progress meter
        Args:
            amount (int): 0-49 fills the right eye, 50-100 also covers left
        """
        if percentage < 0 or percentage > 100:
            raise ValueError('percentage ({}) must be between 0-100'.
                             format(str(percentage)))
        self.bus.emit(Message("enclosure.eyes.fill",
                              {'percentage': percentage}))

    def eyes_brightness(self, level=30):
        """Set the brightness of the eyes in the display.
        Args:
            level (int): 1-30, bigger numbers being brighter
        """
        self.bus.emit(Message("enclosure.eyes.level", {'level': level}))

    def eyes_reset(self):
        """Restore the eyes to their default (ready) state."""
        self.bus.emit(Message("enclosure.eyes.reset"))

    def eyes_spin(self):
        """Make the eyes 'roll'
        """
        self.bus.emit(Message("enclosure.eyes.spin"))

    def eyes_timed_spin(self, length):
        """Make the eyes 'roll' for the given time.
        Args:
            length (int): duration in milliseconds of roll, None = forever
        """
        self.bus.emit(Message("enclosure.eyes.timedspin",
                              {'length': length}))

    def eyes_volume(self, volume):
        """Indicate the volume using the eyes
        Args:
            volume (int): 0 to 11
        """
        if volume < 0 or volume > 11:
            raise ValueError('volume ({}) must be between 0-11'.
                             format(str(volume)))
        self.bus.emit(Message("enclosure.eyes.volume", {'volume': volume}))

    def mouth_reset(self):
        """Restore the mouth display to normal (blank)"""
        self.bus.emit(Message("enclosure.mouth.reset"))

    def mouth_talk(self):
        """Show a generic 'talking' animation for non-synched speech"""
        self.bus.emit(Message("enclosure.mouth.talk"))

    def mouth_think(self):
        """Show a 'thinking' image or animation"""
        self.bus.emit(Message("enclosure.mouth.think"))

    def mouth_listen(self):
        """Show a 'thinking' image or animation"""
        self.bus.emit(Message("enclosure.mouth.listen"))

    def mouth_smile(self):
        """Show a 'smile' image or animation"""
        self.bus.emit(Message("enclosure.mouth.smile"))

    def mouth_viseme(self, start, viseme_pairs):
        """ Send mouth visemes as a list in a single message.

            Arguments:
                start (int):    Timestamp for start of speech
                viseme_pairs:   Pairs of viseme id and cumulative end times
                                (code, end time)

                                codes:
                                 0 = shape for sounds like 'y' or 'aa'
                                 1 = shape for sounds like 'aw'
                                 2 = shape for sounds like 'uh' or 'r'
                                 3 = shape for sounds like 'th' or 'sh'
                                 4 = neutral shape for no sound
                                 5 = shape for sounds like 'f' or 'v'
                                 6 = shape for sounds like 'oy' or 'ao'
        """
        self.bus.emit(Message("enclosure.mouth.viseme_list",
                              {"start": start, "visemes": viseme_pairs}))

    def mouth_text(self, text=""):
        """Display text (scrolling as needed)
        Args:
            text (str): text string to display
        """
        self.bus.emit(Message("enclosure.mouth.text", {'text': text}))

    def mouth_display(self, img_code="", x=0, y=0, refresh=True):
        """Display images on faceplate.
        Args:
            img_code (str): text string that encodes a black and white image
            x (int): x offset for image
            y (int): y offset for image
            refresh (bool): specify whether to clear the faceplate before
                            displaying the new image or not.
                            Useful if you'd like to display multiple images
                            on the faceplate at once.
        """
        self.bus.emit(Message('enclosure.mouth.display',
                              {'img_code': img_code,
                               'xOffset': x,
                               'yOffset': y,
                               'clearPrev': refresh}))

    def mouth_display_png(self, image_absolute_path,
                          invert=False, x=0, y=0, refresh=True):
        """ Send an image to the enclosure.

        Args:
            image_absolute_path (string): The absolute path of the image
            invert (bool): inverts the image being drawn.
            x (int): x offset for image
            y (int): y offset for image
            refresh (bool): specify whether to clear the faceplate before
                            displaying the new image or not.
                            Useful if you'd like to display muliple images
                            on the faceplate at once.
            """
        self.bus.emit(Message("enclosure.mouth.display_image",
                              {'img_path': image_absolute_path,
                               'xOffset': x,
                               'yOffset': y,
                               'invert': invert,
                               'clearPrev': refresh}))

    def weather_display(self, img_code, temp):
        """Show a the temperature and a weather icon

        Args:
            img_code (char): one of the following icon codes
                         0 = sunny
                         1 = partly cloudy
                         2 = cloudy
                         3 = light rain
                         4 = raining
                         5 = stormy
                         6 = snowing
                         7 = wind/mist
            temp (int): the temperature (either C or F, not indicated)
        """
        self.bus.emit(Message("enclosure.weather.display",
                              {'img_code': img_code, 'temp': temp}))

    def activate_mouth_events(self):
        """Enable movement of the mouth with speech"""
        self.bus.emit(Message('enclosure.mouth.events.activate'))

    def deactivate_mouth_events(self):
        """Disable movement of the mouth with speech"""
        self.bus.emit(Message('enclosure.mouth.events.deactivate'))

    def get_eyes_color(self):
        """
        Get the eye RGB color for all pixels

        :returns pixels (list) - list of (r,g,b) tuples for each eye pixel

        """
        message = Message("enclosure.eyes.rgb.get",
                          context={"source": "enclosure_api",
                                   "destination": "enclosure"})
        response = self.bus.wait_for_response(message, "enclosure.eyes.rgb")
        if response:
            return response.data["pixels"]
        raise TimeoutError("Enclosure took too long to respond")

    def get_eyes_pixel_color(self, idx):
        """
        Get the RGB color for a specific eye pixel

        :returns (r,g,b) tuples for selected pixel

        """
        if idx < 0 or idx > 23:
            raise ValueError('row ({}) must be between 0-23'.format(str(idx)))
        return self.get_eyes_color()[idx]



import unittest
from ovos_utils.ssml import SSMLBuilder


class TestSSMLhelpers(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.base_utterance = "this is a test of Open Voice OS SSML utils"
        self.base_utterance2 = "creating ssml for usage with text to speech"

    def test_init_flags(self):
        self.assertEqual(
            SSMLBuilder().say(self.base_utterance).build(),
            "<prosody rate='1'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder(speak_tag=True).say(self.base_utterance).build(),
            "<speak>\n<prosody rate='1'>" + self.base_utterance + "</prosody>\n</speak>"
        )
        self.assertEqual(
            SSMLBuilder(ssml_tag=True).say(self.base_utterance).build(),
            "<ssml>\n<prosody rate='1'>" + self.base_utterance + "</prosody>\n</ssml>"
        )
        self.assertEqual(
            SSMLBuilder(speak_tag=True, ssml_tag=True).say(self.base_utterance).build(),
            "<ssml>\n<speak>\n<prosody rate='1'>" + self.base_utterance + "</prosody>\n</speak>\n</ssml>"
        )

    def test_speak_tags(self):
        self.assertEqual(
            SSMLBuilder().say(self.base_utterance).build(),
            "<prosody rate='1'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().say_loud(self.base_utterance).build(),
            "<prosody volume='1.6'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().say_slow(self.base_utterance).build(),
            "<prosody rate='0.4'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().say_fast(self.base_utterance).build(),
            "<prosody rate='1.6'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().say_low_pitch(self.base_utterance).build(),
            "<prosody pitch='-10%'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().say_high_pitch(self.base_utterance).build(),
            "<prosody pitch='+10%'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().say_emphasis(self.base_utterance).build(),
            "<emphasis level='strong'>" + self.base_utterance + "</emphasis>"
        )
        self.assertEqual(
            SSMLBuilder().say_whispered(self.base_utterance).build(),
            "<whispered>" + self.base_utterance + "</whispered>"
        )
        self.assertEqual(
            SSMLBuilder().whisper(self.base_utterance).build(),
            "<whispered>" + self.base_utterance + "</whispered>"
        )
        self.assertEqual(
            SSMLBuilder().sentence(self.base_utterance).build(),
            "<s>" + self.base_utterance + "</s>"
        )
        self.assertEqual(
            SSMLBuilder().paragraph(self.base_utterance).build(),
            "<p>" + self.base_utterance + "</p>"
        )

    def test_amazon_tags(self):
        self.assertEqual(
            SSMLBuilder().say_strong(self.base_utterance).build(),
            "<amazon:effect vocal-tract-length=\"+20%\">" +
            self.base_utterance + "</amazon:effect>"
        )
        self.assertEqual(
            SSMLBuilder().say_weak(self.base_utterance).build(),
            "<amazon:effect vocal-tract-length=\"-20%\">" +
            self.base_utterance + "</amazon:effect>"
        )
        self.assertEqual(
            SSMLBuilder().say_softly(self.base_utterance).build(),
            "<amazon:effect phonation=\"soft\">" +
            self.base_utterance + "</amazon:effect>"
        )
        self.assertEqual(
            SSMLBuilder().say_auto_breaths(self.base_utterance).build(),
            "<amazon:auto-breaths>" +
            self.base_utterance + "</amazon:auto-breaths>"
        )

    def test_value_tags(self):
        self.assertEqual(
            SSMLBuilder().prosody("XXX", self.base_utterance).build(),
            "<prosody XXX>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().pitch("XXX", self.base_utterance).build(),
            "<prosody pitch='XXX'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().volume("XXX", self.base_utterance).build(),
            "<prosody volume='XXX'>" + self.base_utterance + "</prosody>"
        )
        self.assertEqual(
            SSMLBuilder().volume("XXX", self.base_utterance).build(),
            "<prosody volume='XXX'>" + self.base_utterance + "</prosody>"
        )

        self.assertEqual(
            SSMLBuilder().emphasis("strong", self.base_utterance).build(),
            "<emphasis level='strong'>" + self.base_utterance + "</emphasis>"
        )

    def test_pause(self):
        self.assertEqual(
            SSMLBuilder().pause().build(),
            "<break />"
        )
        self.assertEqual(
            SSMLBuilder().pause(120).build(),
            "<break time=120ms/>"
        )
        self.assertEqual(
            SSMLBuilder().pause(120, "ms").build(),
            "<break time=120ms/>"
        )
        self.assertEqual(
            SSMLBuilder().pause(1, "s").build(),
            "<break time=1s/>"
        )
        self.assertEqual(
            SSMLBuilder().pause_by_strength("XXX").build(),
            "<break strength=xxx/>"
        )

    def test_other(self):
        self.assertEqual(
            SSMLBuilder().audio("XXX", self.base_utterance).build(),
            "<audio src=XXX>" + self.base_utterance + "</audio>"
        )
        # TODO parts_of_speech
        # TODO sub
        # TODO phoneme
        # TODO voice

    def test_multiple(self):
        ssml = SSMLBuilder() \
            .say(self.base_utterance).pause().say_fast(self.base_utterance2) \
            .build()
        self.assertEqual(
            ssml,
            "<prosody rate='1'>" + self.base_utterance +
            "</prosody> <break /> <prosody rate='1.6'>" +
            self.base_utterance2 + "</prosody>"
        )

    def test_tag_parsing(self):
        ssml = SSMLBuilder() \
            .say(self.base_utterance).pause().say_fast(self.base_utterance2) \
            .build()
        self.assertEqual(
            SSMLBuilder.remove_ssml(ssml),
            self.base_utterance + " " + self.base_utterance2
        )
        self.assertEqual(
            SSMLBuilder.extract_ssml_tags(ssml),
            ["<prosody rate='1'>", '</prosody>', '<break />',
             "<prosody rate='1.6'>", '</prosody>']
        )


from ovos_utils.enclosure.mark1.faceplate import FacePlateAnimation, BlackScreen
import copy
import random


# Base animations
# These are mostly meant to be subclassed (empty animations)
class HorizontalScroll(FacePlateAnimation):
    def __init__(self, direction="right", grid=None, bus=None):
        super().__init__(grid, bus)
        assert direction.startswith("r") or direction.startswith("l")
        self.direction = direction[0]

    def animate(self):
        if self.direction == "r":
            self.scroll_right()
        else:
            self.scroll_left()
        if self.is_empty:
            self.stop()


class VerticalScroll(FacePlateAnimation):
    def __init__(self, direction="up",
                 grid=None, bus=None):
        super().__init__(grid, bus)
        assert direction.startswith("u") or direction.startswith("d")
        self.direction = direction[0]

    def animate(self):
        if self.direction == "u":
            self.scroll_up()
        else:
            self.scroll_down()
        if self.is_empty:
            self.stop()


class LeftRight(FacePlateAnimation):
    def __init__(self, direction="right", start="left", grid=None, bus=None):
        super().__init__(grid, bus)
        assert direction.startswith("r") or direction.startswith("l")
        self.direction = direction[0]

        # start at right/left side/center
        inverted = not isinstance(self, BlackScreen)
        if start[0] == "l":
            # left side
            for y in range(self.height):
                for x in range(self.width):
                    if not inverted and self.grid[y][x] == 1:
                        pass
                    elif inverted and self.grid[y][x] == 0:
                        pass
        elif start[0] == "r":
            pass # right side
        else:
            pass # center
        print(self.grid[1])

    def animate(self):
        left_collision = False
        right_collision = False
        inverted = not isinstance(self, BlackScreen)
        for y in range(self.height):
            if inverted:
                if self.grid[y][self.width - 1] == 0:
                    right_collision = True
                if self.grid[y][0] == 0:
                    left_collision = True
            else:
                if self.grid[y][self.width - 1] == 1:
                    right_collision = True
                if self.grid[y][0] == 1:
                    left_collision = True
        if left_collision and right_collision:
            return  # No space left to animate
        elif right_collision:
            self.direction = "l"
        elif left_collision:
            self.direction = "r"
        if self.direction == "r":
            self.scroll_right()
        else:
            self.scroll_left()
        if self.is_empty:
            self.stop()


class UpDown(FacePlateAnimation):
    def __init__(self, direction="up", grid=None, bus=None):
        super().__init__(grid, bus)
        assert direction.startswith("u") or direction.startswith("d")
        self.direction = direction[0]

    def animate(self):
        top_collision = False
        bottom_collision = False
        for x in range(self.width):
            if self.grid[0][x] == 1:
                top_collision = True
            if self.grid[self.height - 1][x] == 1:
                bottom_collision = True

        if top_collision and bottom_collision:
            return  # No space left to animate
        elif top_collision:
            self.direction = "d"
        elif bottom_collision:
            self.direction = "u"
        if self.direction == "u":
            self.scroll_up()
        else:
            self.scroll_down()
        if self.is_empty:
            self.stop()


class CollisionBox(FacePlateAnimation):
    def __init__(self,
                 horizontal_direction=None,
                 vertical_direction=None,
                 grid=None, bus=None):
        super().__init__(grid, bus)
        assert horizontal_direction is None or \
               horizontal_direction.startswith("r") or \
               horizontal_direction.startswith("l")
        assert vertical_direction is None or \
               vertical_direction.startswith("u") or \
               vertical_direction.startswith("d")
        self.vertical_direction = vertical_direction[0] if \
            vertical_direction else None
        self.horizontal_direction = horizontal_direction[0] if \
            horizontal_direction else None

    def animate(self):
        left_collision = False
        right_collision = False
        top_collision = False
        bottom_collision = False
        for y in range(self.height):
            if self.grid[y][self.width - 1] == 1:
                right_collision = True
            if self.grid[y][0] == 1:
                left_collision = True
        for x in range(self.width):
            if self.grid[0][x] == 1:
                top_collision = True
            if self.grid[self.height - 1][x] == 1:
                bottom_collision = True

        if top_collision and bottom_collision:
            self.vertical_direction = None
        elif top_collision:
            self.vertical_direction = "d"
        elif bottom_collision:
            self.vertical_direction = "u"

        if left_collision and right_collision:
            self.horizontal_direction = None
        elif right_collision:
            self.horizontal_direction = "l"
        elif left_collision:
            self.horizontal_direction = "r"

        if self.vertical_direction is None:
            pass
        elif self.vertical_direction == "u":
            self.scroll_up()
        elif self.vertical_direction == "d":
            self.scroll_down()

        if self.horizontal_direction is None:
            pass
        elif self.horizontal_direction == "r":
            self.scroll_right()
        elif self.horizontal_direction == "l":
            self.scroll_left()

        if self.is_empty:
            self.stop()


# Ready to use animations
class SquareWave(HorizontalScroll):
    def __init__(self, direction="r", frequency=3,
                 amplitude=4, grid=None, bus=None):
        super().__init__(direction, grid, bus)
        # frequency must be > 1
        # frequency is in number of pixels
        assert 0 < frequency
        # amplitude must be 2, 4 or 6. else it renders badly
        # amplitude is in number of pixels
        assert 0 < amplitude < self.height
        assert divmod(amplitude, 2)[1] == 0

        self.freq = frequency
        self.amplitude = self.height - amplitude

        self._initial_grid()
        self.invert()

    def _initial_grid(self):
        # draws the initial state
        count = 0
        top = True
        a = self.amplitude // 2 - 1
        for x in range(self.width):
            if top:
                self.grid[a + 1][x] = 1
            else:
                self.grid[-a - 1][x] = 1

            for y in range(self.height):
                if count == 0:
                    self.grid[y][x] = 1
                if y <= a:
                    self.grid[y][x] = 0
                elif y >= self.height - a:
                    self.grid[y][x] = 0
            count += 1
            if count == self.freq + 1:
                count = 0
                top = not top


class StrayDot(CollisionBox):
    def __init__(self,
                 start_x=None,
                 start_y=None,
                 horizontal_direction=None,
                 vertical_direction=None,
                 grid=None, bus=None):
        horizontal_direction = horizontal_direction or \
                               random.choice(["l", "r"])
        vertical_direction = vertical_direction or \
                             random.choice(["u", "d"])
        super().__init__(horizontal_direction, vertical_direction,
                         grid, bus)
        start_x = start_x or random.randint(0, self.width - 1)
        start_y = start_y or random.randint(0, self.height - 1)
        self.grid[start_y][start_x] = 1


class ParticleBox(FacePlateAnimation):
    def __init__(self, n_particles=5, bus=None):
        super().__init__(bus=bus)
        assert 0 < n_particles < 11
        self.n_particles = n_particles
        self.particles = []

        class Dot:
            def __init__(self, idx, x, y, vx, vy):
                self.x = x
                self.y = y
                self.vx = vx
                self.vy = vy
                self.idx = idx

        for i in range(n_particles):
            vx = random.choice(["l", "r"])
            vy = random.choice(["u", "d"])
            x = random.randint(0, self.width - 1)
            y = random.randint(0, self.height - 1)
            while self.grid[y][x] == 1:
                # 2 particles can't occupy same space
                x = random.randint(0, self.width - 1)
                y = random.randint(0, self.height - 1)
            self.grid[y][x] = 1
            self.particles.append(Dot(i, x, y, vx, vy))

    def render_particles(self):
        self.clear()
        for p in self.particles:
            self.grid[p.y][p.x] = 1

    def get_particle(self, x, y):
        for p in self.particles:
            if p.x == x and p.y == y:
                return p

    def process_collisions(self):
        # new particles after this turn
        new_particles = copy.deepcopy(self.particles)

        # NOTE this is not a physics simulation!
        # while it is behaving like an elastic collision
        # if there is a 3+ particle collision results will be incorrect
        # as long as only 2 particles collide it looks accurate
        # max number of particles limited to 10 to minimize chance of this
        # happening
        for p in self.particles:
            idx = p.idx

            # horizontal movement
            if p.vx is None:
                # not moving horizontally
                if p.x != 0:
                    # check for collisions from left p2 -> p1
                    p2 = self.get_particle(p.x - 1, p.y)
                    if p2 and p2.vx == "r":
                        # collision p2 -> p1
                        if p.x == self.width - 1:
                            new_particles[idx].vx = None
                        else:
                            new_particles[idx].vx = "r"
                            new_particles[idx].x += 1
                if p.x != self.width - 1:
                    # check for collisions from right p1 <- p2
                    p2 = self.get_particle(p.x + 1, p.y)
                    if p2 and p2.vx == "l":
                        # collision p1 <- p2
                        if p.x == 0:
                            new_particles[idx].vx = None
                        else:
                            new_particles[idx].vx = "l"
                            new_particles[idx].x -= 1
            # moving right
            elif p.vx == "r":
                p2 = self.get_particle(p.x + 1, p.y)
                if p.x == self.width - 1:
                    # border collision p1 -> |
                    new_particles[idx].vx = "l"
                    new_particles[idx].x -= 1
                elif p2:
                    # particle collision p1 -> p2
                    if p2.vx is None:
                        # p2 moves, p1 stops
                        new_particles[idx].vx = None
                    elif p2.vx == "r":
                        # moving together
                        new_particles[idx].x += 1
                    elif p2 and p2.vx == "l":
                        if p.x == 0:
                            new_particles[idx].vx = None
                        else:
                            # both change direction
                            new_particles[idx].vx = "l"
                            new_particles[idx].x -= 1
                else:
                    # move right
                    new_particles[idx].x += 1
            # moving left
            elif p.vx == "l":
                p2 = self.get_particle(p.x - 1, p.y)
                if p.x == 0:
                    # border collision | <- p1
                    new_particles[idx].vx = "r"
                    new_particles[idx].x += 1
                elif p2:
                    # particle collision  p2 <- p1

                    if p2.vx is None:
                        # p2 moves, p1 stops
                        new_particles[idx].vx = None
                    elif p2.vx == "l":
                        # moving together, no collision
                        new_particles[idx].x -= 1
                    elif p2.vx == "r":
                        if p.x == self.width - 1:
                            new_particles[idx].vx = None
                        else:
                            # both change direction
                            new_particles[idx].vx = "r"
                            new_particles[idx].x += 1
                else:
                    # move left
                    new_particles[idx].x -= 1

            # vertical movement
            if p.vy is None:
                # not moving vertically
                if p.y != 0:
                    # check for collisions from top p2 -> p1
                    p2 = self.get_particle(p.x, p.y - 1)
                    if p2 and p2.vy == "d":
                        if p.y == self.height - 1:
                            new_particles[idx].vy = None
                        else:
                            # collision p2 -> p1
                            new_particles[idx].vy = "d"
                            new_particles[idx].y += 1
                if p.y != self.height - 1:
                    # check for collisions from bottom p1 <- p2
                    p2 = self.get_particle(p.x, p.y + 1)
                    if p2 and p2.vy == "u":
                        # collision p1 <- p2
                        if p.y == 0:  # on top
                            new_particles[idx].vy = None
                        else:
                            new_particles[idx].vy = "u"
                            new_particles[idx].y -= 1
            # moving down
            elif p.vy == "d":
                p2 = self.get_particle(p.x, p.y + 1)
                if p.y == self.height - 1:
                    # border collision p1 -> |
                    new_particles[idx].vy = "u"
                    new_particles[idx].y -= 1
                elif p2:
                    # particle collision p1 -> p2

                    if p2.vy is None:
                        # p2 moves, p1 stops
                        new_particles[idx].vy = None
                    elif p2.vy == "d":
                        # moving together
                        new_particles[idx].y += 1
                    elif p2.vy == "u":
                        if p.y == 0:
                            new_particles[
                                idx].vy = None  # wall absorbed momentum
                        else:
                            # both change direction
                            new_particles[idx].vy = "u"
                            new_particles[idx].y -= 1
                else:
                    # move down
                    new_particles[idx].y += 1
            # moving up
            elif p.vy == "u":

                p2 = self.get_particle(p.x, p.y - 1)
                if p.y == 0:
                    # border collision | <- p1
                    new_particles[idx].vy = "d"
                    new_particles[idx].y += 1
                elif p2:
                    # particle collision  p2 <- p1
                    if p2.vy is None:
                        # p2 moves, p1 stops
                        new_particles[idx].vy = None
                    elif p2.vy == "u":
                        # moving together, no collision
                        new_particles[idx].y -= 1
                    elif p2.vy == "d":
                        # both change direction
                        if p.y == self.height - 1:
                            new_particles[idx].vy = None
                        else:
                            new_particles[idx].vy = "d"
                            new_particles[idx].y += 1
                else:
                    # move left
                    new_particles[idx].y -= 1

        # update processed particles
        self.particles = new_particles

    def animate(self):
        self.process_collisions()
        self.render_particles()


class FallingDots(FacePlateAnimation):
    def __init__(self, n=10, bus=None):
        super().__init__(bus=bus)
        self._create = True
        assert 0 < n < 32
        self.n = n

    @property
    def n_dots(self):
        n = 0
        for y in range(self.height):
            for x in range(self.width):
                if self.grid[y][x]:
                    n += 1
        return n

    def animate(self):
        self.move_down()
        if self._create:
            if random.choice([True, False]):
                self._create = False
                x = random.randint(0, self.width - 1)
                self.grid[0][x] = 1
        if self.n_dots < self.n:
            self._create = True


class StraightParticleShooter(FacePlateAnimation):
    def __init__(self, period=None, bus=None):
        super().__init__(bus=bus)
        self.direction = "d"
        self.period = period
        self.counter = 0
        # draw shooter
        self.grid[0][0] = 1
        self.grid[1][0] = 1
        self.grid[1][1] = 1
        self.grid[2][0] = 1

    def line_down(self):
        old = copy.deepcopy(self.grid)
        for y in range(self.height):
            self.grid[y][0] = old[y - 1][0]
            self.grid[y][1] = old[y - 1][1]

    def line_up(self):
        old = copy.deepcopy(self.grid)
        for y in range(self.height):
            if y == self.height - 1:
                self.grid[y][0] = old[0][0]
                self.grid[y][1] = old[0][1]
            else:
                self.grid[y][0] = old[y + 1][0]
                self.grid[y][1] = old[y + 1][1]

    def scroll_particles(self):
        old = copy.deepcopy(self.grid)
        for x in range(2, self.width):
            for y in range(self.height):
                if old[y][x] == 1:
                    self.grid[y][x] = 0
                    if x < self.width - 1:
                        self.grid[y][x + 1] = 1

    @property
    def num_particles(self):
        n = 0
        for x in range(2, self.width):
            for y in range(self.height):
                if self.grid[y][x] == 1:
                    n += 1
        return n

    @property
    def line(self):
        for y in range(self.height):
            if self.grid[y][0] == 1:
                return y
        return 0

    def animate(self):
        # collision detection
        top_collision = False
        bottom_collision = False
        if self.grid[0][0] == 1:
            top_collision = True
        elif self.grid[self.height - 1][0] == 1:
            bottom_collision = True
        if top_collision:
            self.direction = "d"
        elif bottom_collision:
            self.direction = "u"

        # bounce the "emitter" up and down
        if self.direction == "u":
            self.line_up()
        else:
            self.line_down()

        # create particles
        period = self.period or random.randint(0, 20)
        if self.num_particles < 1 or self.counter >= period:
            self.grid[self.line + 1][2] = 1
            self.counter = 0

        # animate particles
        self.scroll_particles()
        self.counter += 1



if __name__ == "__main__":
    from ovos_utils.messagebus import get_mycroft_bus
    from time import sleep

    bus = get_mycroft_bus("192.168.1.70")

    for faceplate in ParticleBox(bus=bus):
        faceplate.display(invert=False)
        sleep(0.5)


import unittest
from ovos_utils.colors import Color, ColorOutOfSpace


class TestColorHelpers(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.red = Color.from_name("red")
        cls.green = Color.from_name("green")
        cls.light_blue = Color.from_name("light blue")
        cls.white = Color.from_name("whitee")
        cls.undefined_color = ColorOutOfSpace()
        cls.unnamed_color = Color.from_rgb(120, 224, 4)
        cls.artificial_color = Color.from_name("black swamp green")
        cls.constructed = ColorOutOfSpace()
        cls.constructed.set_blue(0.4)
        cls.constructed.set_green(0.1)
        cls.constructed.set_luminance(0.15)

    def test_color_formats(self):
        self.assertEqual(self.white.hex, "#fff")
        self.assertEqual(self.red.hex, "#f00")

        self.assertEqual(self.white.rgb255, (255, 255, 255))
        self.assertEqual(self.red.rgb255, (255, 0, 0))

        self.assertEqual(self.red.hsv, (0.0, 1.0, 1.0))
        self.assertEqual(self.white.hsv, (0.0, 0.0, 1.0))

        self.assertEqual(self.red.hls, (0.0, 0.5, 1.0))
        self.assertEqual(self.white.hls, (0.0, 1.0, 0.0))

    def test_constructors(self):
        self.assertEqual(self.white, Color("white"))
        self.assertEqual(self.white, Color.from_rgb(255, 255, 255))
        self.assertEqual(self.white, Color.from_rgb_percent(1, 1, 1))
        self.assertEqual(self.white, Color.from_hex("#fff"))
        # TODO FIXME
        #self.assertEqual(self.white, Color.from_hsv(0.0, 0.0, 1.0))
        #self.assertEqual(self.white, Color.from_hls(0.0, 1.0, 0.0))

    def test_color_properties(self):
        self.assertEqual(self.red.name, "red")
        self.assertEqual(self.red.color_description, "red")
        self.assertEqual(self.light_blue.color_description, "light blue")

        self.assertEqual(self.unnamed_color.name, None)
        self.assertEqual(self.unnamed_color.color_description,
                         "yellow-ish intense orange color")
        self.assertEqual(self.unnamed_color.main_color, Color("orange"))

        self.assertEqual(self.artificial_color.hue, 0.33)
        self.assertEqual(self.artificial_color.main_color, Color("gray"))
        self.assertEqual(self.artificial_color.name, self.artificial_color.hex)
        self.assertEqual(self.artificial_color.color_description,
                         "dark red-ish black-ish gray color")

        self.assertEqual(self.constructed.main_color, Color("black"))
        self.assertEqual(self.constructed.color_description,
                         "dark blue-ish black-ish intense black color")
        self.assertEqual(self.constructed.name, self.constructed.hex)

    def test_undefined_color(self):
        empty_color = ColorOutOfSpace()
        self.assertEqual(empty_color.name, empty_color.hex)
        self.assertEqual(empty_color.hex, "#000")
        self.assertEqual(empty_color.rgb255, (0, 0, 0))
        self.assertEqual(empty_color.color_description, "black")
        self.assertEqual(empty_color.main_color, Color("gray"))





import json
import os
from os import makedirs
from os.path import isfile, exists, expanduser, join, dirname, isdir

from xdg import BaseDirectory as XDG

from ovos_utils.fingerprinting import core_supports_xdg
from ovos_utils.json_helper import merge_dict, load_commented_json
from ovos_utils.log import LOG
from ovos_utils.system import search_mycroft_core_location

# for downstream support, all XDG paths should respect this
_BASE_FOLDER = "mycroft"
_CONFIG_FILE_NAME = "mycroft.conf"

_DEFAULT_CONFIG = None
_SYSTEM_CONFIG = os.environ.get('MYCROFT_SYSTEM_CONFIG',
                                f'/etc/{_BASE_FOLDER}/{_CONFIG_FILE_NAME}')
# Make sure we support the old location until mycroft moves to XDG
_OLD_USER_CONFIG = join(expanduser('~'), '.' + _BASE_FOLDER, _CONFIG_FILE_NAME)
_USER_CONFIG = join(XDG.xdg_config_home, _BASE_FOLDER, _CONFIG_FILE_NAME)
_WEB_CONFIG_CACHE = join(XDG.xdg_config_home, _BASE_FOLDER, 'web_cache.json')


def get_xdg_base():
    global _BASE_FOLDER
    return _BASE_FOLDER


def set_xdg_base(folder_name):
    global _BASE_FOLDER, _WEB_CONFIG_CACHE
    LOG.info(f"XDG base folder set to: '{folder_name}'")
    _BASE_FOLDER = folder_name
    _WEB_CONFIG_CACHE = join(XDG.xdg_config_home, _BASE_FOLDER,
                             'web_cache.json')


def set_config_filename(file_name, core_folder=None):
    global _CONFIG_FILE_NAME, _SYSTEM_CONFIG, _OLD_USER_CONFIG, _USER_CONFIG, \
        _BASE_FOLDER
    if core_folder:
        _BASE_FOLDER = core_folder
        set_xdg_base(core_folder)
    LOG.info(f"config filename set to: '{file_name}'")
    _CONFIG_FILE_NAME = file_name
    _SYSTEM_CONFIG = os.environ.get('MYCROFT_SYSTEM_CONFIG',
                                    f'/etc/{_BASE_FOLDER}/{_CONFIG_FILE_NAME}')
    # Make sure we support the old location still
    # Deprecated and will be removed eventually
    _OLD_USER_CONFIG = join(expanduser('~'), '.' + _BASE_FOLDER,
                            _CONFIG_FILE_NAME)
    _USER_CONFIG = join(XDG.xdg_config_home, _BASE_FOLDER, _CONFIG_FILE_NAME)


def set_default_config(file_path=None):
    global _DEFAULT_CONFIG
    _DEFAULT_CONFIG = file_path or find_default_config()
    LOG.info(f"default config file changed to: {file_path}")


def find_default_config():
    if _DEFAULT_CONFIG:
        # previously set, otherwise None
        return _DEFAULT_CONFIG
    mycroft_root = search_mycroft_core_location()
    if not mycroft_root:
        raise FileNotFoundError("Couldn't find mycroft core root folder.")
    return join(mycroft_root, _BASE_FOLDER, "configuration", _CONFIG_FILE_NAME)


def find_user_config():
    # ideally it will have been set by downstream using util methods
    old, path = get_config_locations(default=False, web_cache=False,
                                     system=False, old_user=True,
                                     user=True)
    if isfile(path):
        return path

    if core_supports_xdg():
        path = join(XDG.xdg_config_home, _BASE_FOLDER, _CONFIG_FILE_NAME)
    else:
        path = old
        # mark1 runs as a different user
        sysconfig = MycroftSystemConfig()
        platform_str = sysconfig.get("enclosure", {}).get("platform", "")
        if platform_str == "mycroft_mark_1":
            path = "/home/mycroft/.mycroft/mycroft.conf"

    if not isfile(path) and isfile(old):
        # xdg might be disabled in HolmesV compatibility mode
        # or migration might be in progress
        # (user action required when updated from a no xdg install)
        path = old

    return path


def get_config_locations(default=True, web_cache=True, system=True,
                         old_user=True, user=True):
    locs = []
    if default:
        locs.append(_DEFAULT_CONFIG)
    if system:
        locs.append(_SYSTEM_CONFIG)
    if web_cache:
        locs.append(_WEB_CONFIG_CACHE)
    if old_user:
        locs.append(_OLD_USER_CONFIG)
    if user:
        locs.append(_USER_CONFIG)

    return locs


def get_webcache_location():
    return join(XDG.xdg_config_home, _BASE_FOLDER, _CONFIG_FILE_NAME)


def get_xdg_config_locations():
    # This includes both the user config and
    # /etc/xdg/mycroft/mycroft.conf
    xdg_paths = list(reversed(
        [join(p, get_config_filename())
         for p in XDG.load_config_paths(get_xdg_base())]
    ))
    return xdg_paths


def get_config_filename():
    return _CONFIG_FILE_NAME


def set_config_name(name, core_folder=None):
    # TODO deprecate, was only out in a couple versions
    # renamed to match HolmesV
    set_config_filename(name, core_folder)


def read_mycroft_config():
    conf = LocalConf(None)
    conf.merge(MycroftDefaultConfig())
    conf.merge(MycroftSystemConfig())
    conf.merge(MycroftUserConfig())
    return conf


def update_mycroft_config(config, path=None):
    if path is None:
        conf = MycroftUserConfig()
    else:
        conf = LocalConf(path)
    conf.merge(config)
    conf.store()
    return conf


class LocalConf(dict):
    """
        Config dict from file.
    """
    allow_overwrite = True

    def __init__(self, path):
        super(LocalConf, self).__init__()
        self.path = path
        if self.path:
            self.load_local(self.path)

    def load_local(self, path):
        """
            Load local json file into self.

            Args:
                path (str): file to load
        """
        path = expanduser(path)
        if exists(path) and isfile(path):
            try:
                config = load_commented_json(path)
                for key in config:
                    self.__setitem__(key, config[key])

                LOG.debug("Configuration {} loaded".format(path))
            except Exception as e:
                LOG.error("Error loading configuration '{}'".format(path))
                LOG.error(repr(e))
        else:
            LOG.debug("Configuration '{}' not defined, skipping".format(path))

    def reload(self):
        self.load_local(self.path)

    def store(self, path=None):
        """
            store the configuration locally.
        """
        path = path or self.path
        if not path:
            LOG.warning("config path not set, updating user config!!")
            update_mycroft_config(self)
            return
        path = expanduser(path)
        if not isdir(dirname(path)):
            makedirs(dirname(path))
        with open(path, 'w', encoding="utf-8") as f:
            json.dump(self, f, indent=4, ensure_ascii=False)

    def merge(self, conf):
        merge_dict(self, conf)
        return self


class ReadOnlyConfig(LocalConf):
    """ read only  """

    def __init__(self, path, allow_overwrite=False):
        super().__init__(path)
        self.allow_overwrite = allow_overwrite

    def reload(self):
        old = self.allow_overwrite
        self.allow_overwrite = True
        super().reload()
        self.allow_overwrite = old

    def __setitem__(self, key, value):
        if not self.allow_overwrite:
            raise PermissionError
        super().__setitem__(key, value)

    def __setattr__(self, key, value):
        if not self.allow_overwrite:
            raise PermissionError
        super().__setattr__(key, value)

    def merge(self, conf):
        if not self.allow_overwrite:
            raise PermissionError
        super().merge(conf)

    def store(self, path=None):
        if not self.allow_overwrite:
            raise PermissionError
        super().store(path)


class MycroftUserConfig(LocalConf):
    def __init__(self):
        path = find_user_config()
        super().__init__(path)


class MycroftDefaultConfig(ReadOnlyConfig):
    def __init__(self):
        path = find_default_config()
        super().__init__(path)
        if not self.path or not isfile(self.path):
            LOG.error("mycroft root path not found, could not load default "
                      ".conf")

    def set_root_config_path(self, root_config):
        # in case we got it wrong / non standard
        self.path = root_config
        self.reload()


class MycroftSystemConfig(ReadOnlyConfig):
    def __init__(self, allow_overwrite=False):
        path = get_config_locations(default=False, web_cache=False,
                                    system=True, old_user=False,
                                    user=False)[0]
        super().__init__(path, allow_overwrite)


class MycroftXDGConfig(LocalConf):
    def __init__(self):
        path = get_config_locations(default=False, web_cache=False,
                                    system=False, old_user=False,
                                    user=True)[0]
        super().__init__(path)



from ovos_utils.log import LOG
from ovos_utils.messagebus import get_mycroft_bus, Message
import time
from os.path import isfile


class IntentQueryApi:
    """
    Query Intent Service at runtime
    """

    def __init__(self, bus=None, timeout=5):
        self.bus = bus or get_mycroft_bus()
        self.timeout = timeout

    def get_adapt_intent(self, utterance, lang="en-us"):
        """ get best adapt intent for utterance """
        msg = Message("intent.service.adapt.get",
                      {"utterance": utterance, "lang": lang},
                      context={"destination": "intent_service",
                               "source": "intent_api"})

        resp = self.bus.wait_for_response(msg,
                                          'intent.service.adapt.reply',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        return data["intent"]

    def get_padatious_intent(self, utterance, lang="en-us"):
        """ get best padatious intent for utterance """
        msg = Message("intent.service.padatious.get",
                      {"utterance": utterance, "lang": lang},
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        resp = self.bus.wait_for_response(msg,
                                          'intent.service.padatious.reply',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        return data["intent"]

    def get_intent(self, utterance, lang="en-us"):
        """ get best intent for utterance """
        msg = Message("intent.service.intent.get",
                      {"utterance": utterance, "lang": lang},
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        resp = self.bus.wait_for_response(msg,
                                          'intent.service.intent.reply',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        return data["intent"]

    def get_skill(self, utterance, lang="en-us"):
        """ get skill that utterance will trigger """
        intent = self.get_intent(utterance, lang)
        if not intent:
            return None
        # theoretically skill_id might be missing
        if intent.get("skill_id"):
            return intent["skill_id"]
        # retrieve skill from munged intent name
        if intent.get("intent_name"):  # padatious + adapt
            return intent["name"].split(":")[0]
        if intent.get("intent_type"):  # adapt
            return intent["intent_type"].split(":")[0]
        return None  # raise some error here maybe? this should never happen

    def get_skills_manifest(self):
        msg = Message("intent.service.skills.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        resp = self.bus.wait_for_response(msg,
                                          'intent.service.skills.reply',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        return data["skills"]

    def get_active_skills(self, include_timestamps=False):
        msg = Message("intent.service.active_skills.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        resp = self.bus.wait_for_response(msg,
                                          'intent.service.active_skills.reply',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        if include_timestamps:
            return data["skills"]
        # HACK waiting for https://github.com/MycroftAI/mycroft-core/pull/2786
        if len(data["skills"]) and not len(data["skills"][0]) == 2:
            return data["skills"]
        return [s[0] for s in data["skills"]]

    def get_adapt_manifest(self):
        msg = Message("intent.service.adapt.manifest.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        resp = self.bus.wait_for_response(msg,
                                          'intent.service.adapt.manifest',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        return data["intents"]

    def get_padatious_manifest(self):
        msg = Message("intent.service.padatious.manifest.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        resp = self.bus.wait_for_response(msg,
                                          'intent.service.padatious.manifest',
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None
        return data["intents"]

    def get_intent_manifest(self):
        padatious = self.get_padatious_manifest()
        adapt = self.get_adapt_manifest()
        return {"adapt": adapt,
                "padatious": padatious}

    def get_vocab_manifest(self):
        msg = Message("intent.service.adapt.vocab.manifest.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        reply_msg_type = 'intent.service.adapt.vocab.manifest'
        resp = self.bus.wait_for_response(msg,
                                          reply_msg_type,
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None

        vocab = {}
        for voc in data["vocab"]:
            if voc.get("regex"):
                continue
            if voc["end"] not in vocab:
                vocab[voc["end"]] = {"samples": []}
            vocab[voc["end"]]["samples"].append(voc["start"])
        return [{"name": voc, "samples": vocab[voc]["samples"]}
                for voc in vocab]

    def get_regex_manifest(self):
        msg = Message("intent.service.adapt.vocab.manifest.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        reply_msg_type = 'intent.service.adapt.vocab.manifest'
        resp = self.bus.wait_for_response(msg,
                                          reply_msg_type,
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None

        vocab = {}
        for voc in data["vocab"]:
            if not voc.get("regex"):
                continue
            name = voc["regex"].split("(?P<")[-1].split(">")[0]
            if name not in vocab:
                vocab[name] = {"samples": []}
            vocab[name]["samples"].append(voc["regex"])
        return [{"name": voc, "regexes": vocab[voc]["samples"]}
                for voc in vocab]

    def get_entities_manifest(self):
        msg = Message("intent.service.padatious.entities.manifest.get",
                      context={"destination": "intent_service",
                               "source": "intent_api"})
        reply_msg_type = 'intent.service.padatious.entities.manifest'
        resp = self.bus.wait_for_response(msg,
                                          reply_msg_type,
                                          timeout=self.timeout)
        data = resp.data if resp is not None else {}
        if not data:
            LOG.error("Intent Service timed out!")
            return None

        entities = []
        # read files
        for ent in data["entities"]:
            if isfile(ent["file_name"]):
                with open(ent["file_name"]) as f:
                    lines = f.read().replace("(", "").replace(")", "").split(
                        "\n")
                samples = []
                for l in lines:
                    samples += [a.strip() for a in l.split("|") if a.strip()]
                entities.append({"name": ent["name"], "samples": samples})
        return entities

    def get_keywords_manifest(self):
        padatious = self.get_entities_manifest()
        adapt = self.get_vocab_manifest()
        regex = self.get_regex_manifest()
        return {"adapt": adapt,
                "padatious": padatious,
                "regex": regex}


class ConverseTracker:
    """ Using the messagebus this class recreates/keeps track of the state
    of the converse system, it uses both passive listening and active
    queries to sync it's state, it also emits 2 new bus events

    Implements https://github.com/MycroftAI/mycroft-core/pull/1468
    """
    bus = None
    active_skills = []
    converse_timeout = 5  # MAGIC NUMBER  hard coded in mycroft-core
    last_conversed = None
    intent_api = None

    @classmethod
    def connect_bus(cls, mycroft_bus):
        """Registers the bus object to use."""
        # PATCH - in mycroft-core this would be handled in intent_service
        # in here it is done in MycroftSkill.bind so i added this
        # conditional check
        if cls.bus is None and mycroft_bus is not None:
            cls.bus = mycroft_bus
            cls.intent_api = IntentQueryApi(cls.bus)
            cls.register_bus_events()

    @classmethod
    def register_bus_events(cls):
        cls.bus.on('active_skill_request', cls.handle_activate_request)
        cls.bus.on('skill.converse.response', cls.handle_converse_response)
        cls.bus.on("mycroft.skill.handler.start", cls.handle_intent_start)
        cls.bus.on("recognizer_loop:utterance", cls.handle_utterance)

    # public methods
    @classmethod
    def check_skill(cls, skill_id):
        """ Check if a skill is active """
        cls.filter_active_skills()
        for skill in list(cls.active_skills):
            if skill[0] == skill_id:
                return True
        return False

    @classmethod
    def filter_active_skills(cls):
        """ Removes expired skills from active skill list """
        # filter timestamps
        for skill in list(cls.active_skills):
            if time.time() - skill[1] <= cls.converse_timeout * 60:
                cls.remove_active_skill(skill[0])

    @classmethod
    def sync_with_intent_service(cls):
        """sync active skill list using intent api

        WARNING
            we don't have the timestamps so order might be messed up!!
            avoid calling this until
        """
        skill_ids = cls.intent_api.get_active_skills(include_timestamps=True)
        if skill_ids:
            if len(skill_ids[0]) == 2:
                # PR was merged! hurray!
                cls.active_skills = skill_ids
            else:
                # hoping they come sorted by timestamp....
                # older to newer (most recently used)
                for skill_id in reversed(skill_ids):
                    # are we tracking this skill ?
                    if not cls.check_skill(skill_id):
                        # we missed adding this skill in our tracking
                        cls.add_active_skill(skill_id)
                for skill in cls.active_skills:
                    if skill[0] not in skill_ids:
                        # we missed removing this skill in our tracking
                        cls.remove_active_skill(skill[0])

    # https://github.com/MycroftAI/mycroft-core/pull/1468
    @classmethod
    def remove_active_skill(cls, skill_id, silent=False):
        """
        Emits "converse.skill.deactivated" event, improvement of #1468
        """
        for skill in list(cls.active_skills):
            if skill[0] == skill_id:
                cls.active_skills.remove(skill)
                if not silent:
                    cls.bus.emit(Message("converse.skill.deactivated",
                                         {"skill_id": skill[0]}))

    @classmethod
    def add_active_skill(cls, skill_id):
        """
        Emits "converse.skill.activated" event, improvement of #1468
        """
        # search the list for an existing entry that already contains it
        # and remove that reference
        if skill_id != '':
            cls.remove_active_skill(skill_id, silent=True)
            # add skill with timestamp to start of skill_list
            cls.active_skills.insert(0, [skill_id, time.time()])
            # this might be sent more than once and it's perfectly fine
            # it's just a new info message not consumed anywhere by default
            cls.bus.emit(Message("converse.skill.activated",
                                 {"skill_id": skill_id}))
        else:
            LOG.warning('Skill ID was empty, won\'t add to list of '
                        'active skills.')

    # status tracking
    @classmethod
    def handle_activate_request(cls, message):
        """
        a skill bumped itself to the top of active skills list
        duplicate functionality from mycroft-core, keeping list in sync
        """
        skill_id = message.data["skill_id"]
        cls.add_active_skill(skill_id)

    @classmethod
    def handle_converse_error(cls, message):
        """
        a skill was removed from active skill list due to converse error
        duplicate functionality from mycroft-core, keeping list in sync
        """
        skill_id = message.data["skill_id"]
        if message.data["error"] == "skill id does not exist":
            cls.remove_active_skill(skill_id)

    @classmethod
    def handle_intent_start(cls, message):
        """
        duplicate functionality from mycroft-core, keeping list in sync

        TODO skill_id from message, core is not passing it along... it used
        to be possible to retrieve it from munged message but that changed.
        send a PR (if those got merged this code wouldn't exist)

        handle_utterance will take over this functionality for now
        handle_converse_response will take corrective action
        """
        # skill_id = message.data["skill_id"]
        # bump skill to top of active list
        # cls.add_active_skill(skill_id)

    @classmethod
    def handle_utterance(cls, message):
        """
        duplicate functionality from mycroft-core, keeping list in sync

        WORKAROUND - skill_id missing in handle_intent_start, will keep list
        in sync by using the IntentAPI to check what skill the utterance
        should trigger

        handle_converse_response will take corrective action
        """
        # NOTE borked in mycroft-core
        # needs https://github.com/MycroftAI/mycroft-core/pull/2786
        skill_id = cls.intent_api.get_skill(message.data["utterances"][0])
        if skill_id:
            # this skill will trigger and therefore is the last active skill
            cls.add_active_skill(skill_id)
        # will remove expired intents from list
        cls.filter_active_skills()

    @classmethod
    def handle_converse_response(cls, message):
        """
        tracks last_conversed skill

        FAILSAFE - additional checks to correct active skills list,
        but that should never happen, accounts for mistakes in
        handle_utterance / intent_api

        accounts for https://github.com/MycroftAI/mycroft-core/pull/2786
        not yet being merged
        """
        skill_id = message.data["skill_id"]
        if 'error' in message.data:
            cls.handle_converse_error(message)
        elif message.data.get('result') is True:
            cls.last_conversed = skill_id
            if not cls.check_skill(skill_id):
                # seems like we missed adding this skill to active list
                # NOTE this is a failsafe and should never trigger
                # since this answered True we have the real timestamp
                cls.add_active_skill(skill_id)
        elif not cls.check_skill(skill_id):
            # seems like we missed adding this skill to active list
            # NOTE this is a failsafe and should never trigger
            # since this answered false and we don't have the real timestamp
            # let's add it to the end of the active_skills list
            ts = time.time()
            if len(cls.active_skills):
                ts = cls.active_skills[-1][1]
            cls.active_skills.append([skill_id, ts])





import time
from ovos_utils.log import LOG
from ovos_utils.messagebus import get_mycroft_bus


class EnclosureTemplate:
    """
    This base class is intended to be used to interface with the hardware
    that is running Mycroft.  It exposes all possible commands which
    can be sent to a Mycroft enclosure implementation.
    """

    def __init__(self, bus=None, name=""):
        self._mouth_events = False
        self._running = False
        self.bus = bus or get_mycroft_bus()
        self.log = LOG
        self.name = name

        self.bus.on("enclosure.reset", self.on_reset)

        # enclosure commands for Mycroft's Hardware.
        self.bus.on("enclosure.system.reset", self.on_system_reset)
        self.bus.on("enclosure.system.mute", self.on_system_mute)
        self.bus.on("enclosure.system.unmute", self.on_system_unmute)
        self.bus.on("enclosure.system.blink", self.on_system_blink)

        # enclosure commands for eyes
        self.bus.on('enclosure.eyes.on', self.on_eyes_on)
        self.bus.on('enclosure.eyes.off', self.on_eyes_off)
        self.bus.on('enclosure.eyes.blink', self.on_eyes_blink)
        self.bus.on('enclosure.eyes.narrow', self.on_eyes_narrow)
        self.bus.on('enclosure.eyes.look', self.on_eyes_look)
        self.bus.on('enclosure.eyes.color', self.on_eyes_color)
        self.bus.on('enclosure.eyes.level', self.on_eyes_brightness)
        self.bus.on('enclosure.eyes.volume', self.on_eyes_volume)
        self.bus.on('enclosure.eyes.spin', self.on_eyes_spin)
        self.bus.on('enclosure.eyes.timedspin', self.on_eyes_timed_spin)
        self.bus.on('enclosure.eyes.reset', self.on_eyes_reset)
        self.bus.on('enclosure.eyes.setpixel', self.on_eyes_set_pixel)
        self.bus.on('enclosure.eyes.fill', self.on_eyes_fill)

        # enclosure commands for mouth
        self.bus.on("enclosure.mouth.events.activate",
                    self._activate_mouth_events)
        self.bus.on("enclosure.mouth.events.deactivate",
                    self._deactivate_mouth_events)
        self.bus.on("enclosure.mouth.talk", self._on_mouth_talk)
        self.bus.on("enclosure.mouth.think", self._on_mouth_think)
        self.bus.on("enclosure.mouth.listen", self._on_mouth_listen)
        self.bus.on("enclosure.mouth.smile", self._on_mouth_smile)
        self.bus.on("enclosure.mouth.viseme", self._on_mouth_viseme)
        # mouth/matrix display
        self.bus.on("enclosure.mouth.reset", self.on_display_reset)
        self.bus.on("enclosure.mouth.text", self.on_text)
        self.bus.on("enclosure.mouth.display", self.on_display)
        self.bus.on("enclosure.weather.display", self.on_weather_display)

        # audio events
        self.bus.on('recognizer_loop:record_begin', self.on_record_begin)
        self.bus.on('recognizer_loop:record_end', self.on_record_end)
        self.bus.on("recognizer_loop:sleep", self.on_sleep)
        self.bus.on('recognizer_loop:audio_output_start',
                    self.on_audio_output_start)
        self.bus.on('recognizer_loop:audio_output_end',
                    self.on_audio_output_end)

        # other events
        self.bus.on("mycroft.awoken", self.on_awake)
        self.bus.on("speak", self.on_speak)
        self.bus.on("enclosure.notify.no_internet", self.on_no_internet)

        self._activate_mouth_events()

    def shutdown(self):
        """

        """
        self.bus.remove("enclosure.reset", self.on_reset)
        self.bus.remove("enclosure.system.reset", self.on_system_reset)
        self.bus.remove("enclosure.system.mute", self.on_system_mute)
        self.bus.remove("enclosure.system.unmute", self.on_system_unmute)
        self.bus.remove("enclosure.system.blink", self.on_system_blink)

        self.bus.remove("enclosure.eyes.on", self.on_eyes_on)
        self.bus.remove("enclosure.eyes.off", self.on_eyes_off)
        self.bus.remove("enclosure.eyes.blink", self.on_eyes_blink)
        self.bus.remove("enclosure.eyes.narrow", self.on_eyes_narrow)
        self.bus.remove("enclosure.eyes.look", self.on_eyes_look)
        self.bus.remove("enclosure.eyes.color", self.on_eyes_color)
        self.bus.remove("enclosure.eyes.brightness", self.on_eyes_brightness)
        self.bus.remove("enclosure.eyes.reset", self.on_eyes_reset)
        self.bus.remove("enclosure.eyes.timedspin", self.on_eyes_timed_spin)
        self.bus.remove("enclosure.eyes.volume", self.on_eyes_volume)
        self.bus.remove("enclosure.eyes.spin", self.on_eyes_spin)
        self.bus.remove("enclosure.eyes.set_pixel", self.on_eyes_set_pixel)

        self.bus.remove("enclosure.mouth.reset", self.on_display_reset)
        self.bus.remove("enclosure.mouth.talk", self.on_talk)
        self.bus.remove("enclosure.mouth.think", self.on_think)
        self.bus.remove("enclosure.mouth.listen", self.on_listen)
        self.bus.remove("enclosure.mouth.smile", self.on_smile)
        self.bus.remove("enclosure.mouth.viseme", self.on_viseme)
        self.bus.remove("enclosure.mouth.text", self.on_text)
        self.bus.remove("enclosure.mouth.display", self.on_display)
        self.bus.remove("enclosure.mouth.events.activate",
                        self._activate_mouth_events)
        self.bus.remove("enclosure.mouth.events.deactivate",
                        self._deactivate_mouth_events)

        self.bus.remove("enclosure.weather.display", self.on_weather_display)

        self.bus.remove("mycroft.awoken", self.on_awake)
        self.bus.remove("recognizer_loop:sleep", self.on_sleep)
        self.bus.remove("speak", self.on_speak)
        self.bus.remove('recognizer_loop:record_begin', self.on_record_begin)
        self.bus.remove('recognizer_loop:record_end', self.on_record_end)
        self.bus.remove('recognizer_loop:audio_output_start',
                        self.on_audio_output_start)
        self.bus.remove("enclosure.notify.no_internet", self.on_no_internet)

        self._deactivate_mouth_events()
        self._running = False

    def run(self):
        ''' start enclosure '''
        self._running = True
        while self._running:
            time.sleep(1)

    # Audio Events
    def on_record_begin(self, message=None):
        ''' listening started '''
        pass

    def on_record_end(self, message=None):
        ''' listening ended '''
        pass

    def on_audio_output_start(self, message=None):
        ''' speaking started '''
        pass

    def on_audio_output_end(self, message=None):
        ''' speaking started '''
        pass

    def on_awake(self, message=None):
        ''' on wakeup animation '''
        pass

    def on_sleep(self, message=None):
        ''' on naptime animation '''
        # TODO naptime skill animation should be ond here
        pass

    def on_speak(self, message=None):
        ''' on speak messages, intended for enclosures that disregard
        visemes '''
        pass

    def on_reset(self, message=None):
        """The enclosure should restore itself to a started state.
        Typically this would be represented by the eyes being 'open'
        and the mouth reset to its default (smile or blank).
        """
        pass

    # System Events
    def on_no_internet(self, message=None):
        """

        Args:
            message:
        """
        pass

    def on_system_reset(self, message=None):
        """The enclosure hardware should reset any CPUs, etc."""
        pass

    def on_system_mute(self, message=None):
        """Mute (turn off) the system speaker."""
        pass

    def on_system_unmute(self, message=None):
        """Unmute (turn on) the system speaker."""
        pass

    def on_system_blink(self, message=None):
        """The 'eyes' should blink the given number of times.
        Args:
            times (int): number of times to blink
        """
        pass

    # Eyes events
    def on_eyes_on(self, message=None):
        """Illuminate or show the eyes."""
        pass

    def on_eyes_off(self, message=None):
        """Turn off or hide the eyes."""
        pass

    def on_eyes_fill(self, message=None):
        pass

    def on_eyes_blink(self, message=None):
        """Make the eyes blink
        Args:
            side (str): 'r', 'l', or 'b' for 'right', 'left' or 'both'
        """
        pass

    def on_eyes_narrow(self, message=None):
        """Make the eyes look narrow, like a squint"""
        pass

    def on_eyes_look(self, message=None):
        """Make the eyes look to the given side
        Args:
            side (str): 'r' for right
                        'l' for left
                        'u' for up
                        'd' for down
                        'c' for crossed
        """
        pass

    def on_eyes_color(self, message=None):
        """Change the eye color to the given RGB color
        Args:
            r (int): 0-255, red value
            g (int): 0-255, green value
            b (int): 0-255, blue value
        """
        pass

    def on_eyes_brightness(self, message=None):
        """Set the brightness of the eyes in the display.
        Args:
            level (int): 1-30, bigger numbers being brighter
        """
        pass

    def on_eyes_reset(self, message=None):
        """Restore the eyes to their default (ready) state."""
        pass

    def on_eyes_timed_spin(self, message=None):
        """Make the eyes 'roll' for the given time.
        Args:
            length (int): duration in milliseconds of roll, None = forever
        """
        pass

    def on_eyes_volume(self, message=None):
        """Indicate the volume using the eyes
        Args:
            volume (int): 0 to 11
        """
        pass

    def on_eyes_spin(self, message=None):
        """
        Args:
        """
        pass

    def on_eyes_set_pixel(self, message=None):
        """
        Args:
        """
        pass

    # Mouth events
    def _on_mouth_reset(self, message=None):
        """Restore the mouth display to normal (blank)"""
        if self.mouth_events_active:
            self.on_display_reset(message)

    def _on_mouth_talk(self, message=None):
        """Show a generic 'talking' animation for non-synched speech"""
        if self.mouth_events_active:
            self.on_talk(message)

    def _on_mouth_think(self, message=None):
        """Show a 'thinking' image or animation"""
        if self.mouth_events_active:
            self.on_think(message)

    def _on_mouth_listen(self, message=None):
        """Show a 'thinking' image or animation"""
        if self.mouth_events_active:
            self.on_listen(message)

    def _on_mouth_smile(self, message=None):
        """Show a 'smile' image or animation"""
        if self.mouth_events_active:
            self.on_smile(message)

    def _on_mouth_viseme(self, message=None):
        """Display a viseme mouth shape for synched speech
        Args:
            code (int):  0 = shape for sounds like 'y' or 'aa'
                         1 = shape for sounds like 'aw'
                         2 = shape for sounds like 'uh' or 'r'
                         3 = shape for sounds like 'th' or 'sh'
                         4 = neutral shape for no sound
                         5 = shape for sounds like 'f' or 'v'
                         6 = shape for sounds like 'oy' or 'ao'
        """
        if self.mouth_events_active:
            self.on_viseme(message)

    def _on_mouth_text(self, message=None):
        """Display text (scrolling as needed)
        Args:
            text (str): text string to display
        """
        if self.mouth_events_active:
            self.on_text(message)

    def _on_mouth_display(self, message=None):
        if self.mouth_events_active:
            self.on_display(message)

    # Display (faceplate) events
    def on_display_reset(self, message=None):
        """Restore the mouth display to normal (blank)"""
        pass

    def on_talk(self, message=None):
        """Show a generic 'talking' animation for non-synched speech"""
        pass

    def on_think(self, message=None):
        """Show a 'thinking' image or animation"""
        pass

    def on_listen(self, message=None):
        """Show a 'thinking' image or animation"""
        pass

    def on_smile(self, message=None):
        """Show a 'smile' image or animation"""
        pass

    def on_viseme(self, message=None):
        """Display a viseme mouth shape for synched speech
        Args:
            code (int):  0 = shape for sounds like 'y' or 'aa'
                         1 = shape for sounds like 'aw'
                         2 = shape for sounds like 'uh' or 'r'
                         3 = shape for sounds like 'th' or 'sh'
                         4 = neutral shape for no sound
                         5 = shape for sounds like 'f' or 'v'
                         6 = shape for sounds like 'oy' or 'ao'
        """
        pass

    def on_text(self, message=None):
        """Display text (scrolling as needed)
        Args:
            text (str): text string to display
        """
        pass

    def on_display(self, message=None):
        """Display images on faceplate. Currently supports images up to 16x8,
           or half the face. You can use the 'x' parameter to cover the other
           half of the faceplate.
        Args:
            img_code (str): text string that encodes a black and white image
            x (int): x offset for image
            y (int): y offset for image
            refresh (bool): specify whether to clear the faceplate before
                            displaying the new image or not.
                            Useful if you'd like to display muliple images
                            on the faceplate at once.
        """
        pass

    def on_weather_display(self, message=None):
        """Show a the temperature and a weather icon

        Args:
            img_code (char): one of the following icon codes
                         0 = sunny
                         1 = partly cloudy
                         2 = cloudy
                         3 = light rain
                         4 = raining
                         5 = stormy
                         6 = snowing
                         7 = wind/mist
            temp (int): the temperature (either C or F, not indicated)
        """
        pass

    @property
    def mouth_events_active(self):
        return self._mouth_events

    def _activate_mouth_events(self, message=None):
        """Enable movement of the mouth with speech"""
        self._mouth_events = True

    def _deactivate_mouth_events(self, message=None):
        """Disable movement of the mouth with speech"""
        self._mouth_events = False



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from threading import Thread
from time import sleep
import os
from os.path import isdir, join, dirname
import re
import datetime
import kthread
from ovos_utils.network_utils import *
from inflection import camelize, titleize, transliterate, parameterize, \
    ordinalize


def ensure_mycroft_import():
    try:
        import mycroft
    except ImportError:
        import sys
        from ovos_utils import get_mycroft_root
        MYCROFT_ROOT_PATH = get_mycroft_root()
        if MYCROFT_ROOT_PATH is not None:
            sys.path.append(MYCROFT_ROOT_PATH)
        else:
            raise


def resolve_ovos_resource_file(res_name):
    """Convert a resource into an absolute filename.
    used internally for ovos resources
    """
    # First look for fully qualified file (e.g. a user setting)
    if os.path.isfile(res_name):
        return res_name

    # now look in bundled ovos resources
    filename = join(dirname(__file__), "res", res_name)
    if os.path.isfile(filename):
        return filename

    # let's look in ovos_workshop if it's installed
    try:
        import ovos_workshop
        pkg_dir = dirname(ovos_workshop.__file__)
        filename = join(pkg_dir, "res", res_name)
        if os.path.isfile(filename):
            return filename
        filename = join(pkg_dir, "res", "ui", res_name)
        if os.path.isfile(filename):
            return filename
    except:
        pass
    return None  # Resource cannot be resolved


def get_mycroft_root():
    paths = [
        "/opt/venvs/mycroft-core/lib/python3.7/site-packages/",  # mark1/2
        "/opt/venvs/mycroft-core/lib/python3.4/site-packages/ ",  # old mark1 installs
        "/home/pi/mycroft-core"  # picroft
    ]
    for p in paths:
        if isdir(join(p, "mycroft")):
            return p
    return None


def resolve_resource_file(res_name, root_path=None, config=None):
    """Convert a resource into an absolute filename.

    Resource names are in the form: 'filename.ext'
    or 'path/filename.ext'

    The system wil look for ~/.mycroft/res_name first, and
    if not found will look at /opt/mycroft/res_name,
    then finally it will look for res_name in the 'mycroft/res'
    folder of the source code package.

    Example:
    With mycroft running as the user 'bob', if you called
        resolve_resource_file('snd/beep.wav')
    it would return either '/home/bob/.mycroft/snd/beep.wav' or
    '/opt/mycroft/snd/beep.wav' or '.../mycroft/res/snd/beep.wav',
    where the '...' is replaced by the path where the package has
    been installed.

    Args:
        res_name (str): a resource path/name
        config (dict): mycroft.conf, to read data directory from
    Returns:
        str: path to resource or None if no resource found
    """
    if config is None:
        from ovos_utils.configuration import read_mycroft_config
        config = read_mycroft_config()

    # First look for fully qualified file (e.g. a user setting)
    if os.path.isfile(res_name):
        return res_name

    # Now look for ~/.mycroft/res_name (in user folder)
    filename = os.path.expanduser("~/.mycroft/" + res_name)
    if os.path.isfile(filename):
        return filename

    # Next look for /opt/mycroft/res/res_name
    data_dir = os.path.expanduser(config.get('data_dir', "/opt/mycroft"))
    filename = os.path.expanduser(os.path.join(data_dir, res_name))
    if os.path.isfile(filename):
        return filename

    # look in ovos_utils package itself
    found = resolve_ovos_resource_file(res_name)
    if found:
        return found

    # Finally look for it in the source package
    paths = [
        "/opt/venvs/mycroft-core/lib/python3.7/site-packages/",  # mark1/2
        "/opt/venvs/mycroft-core/lib/python3.4/site-packages/ ",  # old mark1 installs
        "/home/pi/mycroft-core"  # picroft
    ]
    if root_path:
        paths += [root_path]
    for p in paths:
        filename = os.path.join(p, 'mycroft', 'res', res_name)
        filename = os.path.abspath(os.path.normpath(filename))
        if os.path.isfile(filename):
            return filename

    return None  # Resource cannot be resolved


def create_killable_daemon(target, args=(), kwargs=None, autostart=True):
    """Helper to quickly create and start a thread with daemon = True"""
    t = kthread.KThread(target=target, args=args, kwargs=kwargs)
    t.daemon = True
    if autostart:
        t.start()
    return t


def create_daemon(target, args=(), kwargs=None, autostart=True):
    """Helper to quickly create and start a thread with daemon = True"""
    t = Thread(target=target, args=args, kwargs=kwargs)
    t.daemon = True
    if autostart:
        t.start()
    return t


def create_loop(target, interval, args=(), kwargs=None):
    """
    Helper to quickly create and start a thread with daemon = True
    and repeat it every interval seconds
    """

    def loop(*args, **kwargs):
        try:
            while True:
                target(*args, **kwargs)
                sleep(interval)
        except KeyboardInterrupt:
            return

    return create_daemon(loop, args, kwargs)


def wait_for_exit_signal():
    """Blocks until KeyboardInterrupt is received"""
    try:
        while True:
            sleep(100)
    except KeyboardInterrupt:
        pass


def get_handler_name(handler):
    """Name (including class if available) of handler function.

    Arguments:
        handler (function): Function to be named

    Returns:
        string: handler name as string
    """
    if '__self__' in dir(handler) and 'name' in dir(handler.__self__):
        return handler.__self__.name + '.' + handler.__name__
    else:
        return handler.__name__


def camel_case_split(identifier: str) -> str:
    """Split camel case string"""
    regex = '.+?(?:(?<=[a-z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z])|$)'
    matches = re.finditer(regex, identifier)
    return ' '.join([m.group(0) for m in matches])


def rotate_list(l, n=1):
    return l[n:] + l[:n]


def datestr2ts(datestr):
    y = int(datestr[:4])
    m = int(datestr[4:6])
    d = int(datestr[-2:])
    dt = datetime.datetime(y, m, d)
    return dt.timestamp()




from os.path import expanduser, isdir, dirname, join
from os import system, makedirs, listdir
from ovos_utils.lang.detect import detect_lang
from ovos_utils.lang.translate import translate_text


def get_tts(sentence, lang="en-us", mp3_file="/tmp/google_tx_tts.mp3"):
    # TODO privacy issues - https://github.com/OpenVoiceOS/ovos_utils/issues/2
    ext = "mp3"
    if not mp3_file.endswith(ext):
        mp3_file += "." + ext
    mp3_file = expanduser(mp3_file)
    if not isdir(dirname(mp3_file)):
        makedirs(dirname(mp3_file))
    get_sentence = 'wget -q -U Mozilla -O' + mp3_file + \
                   ' "https://translate.google.com/translate_tts?tl=' + \
                   lang + '&q=' + sentence + '&client=tw-ob' + '"'
    system(get_sentence)
    return mp3_file


def get_language_dir(base_path, lang="en-us"):
    """ checks for all language variations and returns best path """
    lang_path = join(base_path, lang)
    # base_path/en-us
    if isdir(lang_path):
        return lang_path
    if "-" in lang:
        main = lang.split("-")[0]
        # base_path/en
        general_lang_path = join(base_path, main)
        if isdir(general_lang_path):
            return general_lang_path
    else:
        main = lang
    # base_path/en-uk, base_path/en-au...
    if isdir(base_path):
        candidates = [join(base_path, f)
                      for f in listdir(base_path) if f.startswith(main)]
        paths = [p for p in candidates if isdir(p)]
        # TODO how to choose best local dialect?
        if len(paths):
            return paths[0]
    return join(base_path, lang)
